package cn.fw.valhalla.service.bus;

import cn.fw.valhalla.common.constant.RoleCode;
import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.domain.db.pool.StammkundePool;
import cn.fw.valhalla.domain.db.report.CustomerRetentionRatio;
import cn.fw.valhalla.domain.enums.StammkundeStatusEnum;
import cn.fw.valhalla.rpc.erp.UserService;
import cn.fw.valhalla.rpc.erp.dto.PostUserDTO;
import cn.fw.valhalla.rpc.oop.OopService;
import cn.fw.valhalla.rpc.oop.dto.ShopDTO;
import cn.fw.valhalla.service.data.CustomerRetentionRatioService;
import cn.fw.valhalla.service.data.StammkundePoolService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.sql.Timestamp;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author : kurisu
 * @className : CustomerRetentionRatioBizService
 * @description : 客户保持率
 * @date: 2020-11-24 15:59
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class CustomerRetentionRatioBizService {
    private final OopService oopService;
    private final UserService userService;
    private final CustomerRetentionRatioService customerRetentionRatioService;
    private final StammkundePoolService stammkundePoolService;

    @Transactional(rollbackFor = Exception.class)
    public void extracting(Date nowDate) {
        log.info("开始抽取保有客保持率数据，日期:{}", DateUtil.getStringDateShort(nowDate));
        List<ShopDTO> allShops = oopService.getAllShops(2L);
        if (CollectionUtils.isEmpty(allShops)) {
            return;
        }
        customerRetentionRatioService.removeByDate(DateUtil.startDate(nowDate));
        for (ShopDTO shop : allShops) {
            extractingPerson(shop, nowDate);
        }
    }

    private void extractingPerson(ShopDTO shop, Date nowDate) {
        List<PostUserDTO> userDTOS = userService.getUserByRole(shop.getId(), RoleCode.FWGW);
        if (CollectionUtils.isEmpty(userDTOS)) {
            return;
        }
        Timestamp expiredDay = DateUtil.getExpiredDay(nowDate, -180);
        Date monthEndDay = DateUtil.getMonthEndDay(expiredDay);
        final LocalDate initialDay = DateUtil.date2LocalDate(monthEndDay);

        List<CustomerRetentionRatio> list = new ArrayList<>();
        for (PostUserDTO userDTO : userDTOS) {
            CustomerRetentionRatio ratio = new CustomerRetentionRatio();
            ratio.setDataDate(nowDate);
            ratio.setDimensions(1);
            ratio.setUserId(userDTO.getUserId());
            ratio.setUserName(userDTO.getUserName());
            ratio.setShopId(shop.getId());
            ratio.setShopName(shop.getShortName());
            StammkundePool pool = firstOne(userDTO.getUserId(), shop.getId());
            if (Objects.isNull(pool)) {
                ratio.setInitialNum(0);
                ratio.setDefeatNum(0);
            } else {
                Date startDay = DateUtil.getMonthEndDay(pool.getActivationTime());
                LocalDate acTime = DateUtil.date2LocalDate(startDay);
                if (initialDay.isBefore(acTime)) {
                    if (startDay.after(nowDate)) {
                        startDay = nowDate;
                    }
                    int initialNum = initialNum(userDTO.getUserId(), shop.getId(), startDay);
                    int defeatNum = defeatNum(userDTO.getUserId(), shop.getId(), startDay);
                    ratio.setInitialNum(initialNum);
                    ratio.setDefeatNum(defeatNum);
                } else {
                    int initialNum = initialNum(userDTO.getUserId(), shop.getId(), monthEndDay);
                    int defeatNum = defeatNum(userDTO.getUserId(), shop.getId(), monthEndDay);
                    ratio.setInitialNum(initialNum);
                    ratio.setDefeatNum(defeatNum);
                }
            }
            list.add(ratio);
        }
        customerRetentionRatioService.saveBatch(list);
    }

    private void extractingShop(ShopDTO shop, Date nowDate) {
        Timestamp expiredDay = DateUtil.getExpiredDay(nowDate, -180);
        Date monthEndDay = DateUtil.getMonthEndDay(expiredDay);
        final LocalDate initialDay = DateUtil.date2LocalDate(monthEndDay);

        CustomerRetentionRatio ratio = new CustomerRetentionRatio();
        ratio.setDataDate(nowDate);
        ratio.setDimensions(2);
        ratio.setUserId(-1L);
        ratio.setUserName("");
        ratio.setShopId(shop.getId());
        ratio.setShopName(shop.getShortName());
        StammkundePool pool = firstOne(null, shop.getId());
        if (Objects.isNull(pool)) {
            ratio.setInitialNum(0);
            ratio.setDefeatNum(0);
        } else {
            Date startDay = DateUtil.getMonthEndDay(pool.getActivationTime());
            LocalDate acTime = DateUtil.date2LocalDate(startDay);
            if (initialDay.isBefore(acTime)) {
                if (startDay.after(nowDate)) {
                    startDay = nowDate;
                }
                int initialNum = initialNum(shop.getId(), startDay);
                int defeatNum = defeatNum(shop.getId(), startDay);
                ratio.setInitialNum(initialNum);
                ratio.setDefeatNum(defeatNum);
            } else {
                int initialNum = initialNum(shop.getId(), monthEndDay);
                int defeatNum = defeatNum(shop.getId(), monthEndDay);
                ratio.setInitialNum(initialNum);
                ratio.setDefeatNum(defeatNum);
            }
        }
        customerRetentionRatioService.save(ratio);
    }

    private StammkundePool firstOne(Long userId, Long shopId) {
        return stammkundePoolService.getOne(Wrappers.<StammkundePool>lambdaQuery()
                .eq(Objects.nonNull(userId), StammkundePool::getAdviserId, userId)
                .eq(StammkundePool::getShopId, shopId)
                .orderByAsc(StammkundePool::getActivationTime)
                .last(" limit 1")
        );
    }

    private int initialNum(Long userId, Long shopId, Date expire) {
        return stammkundePoolService.count(Wrappers.<StammkundePool>lambdaQuery()
                .eq(StammkundePool::getAdviserId, userId)
                .eq(StammkundePool::getPoolStatus, StammkundeStatusEnum.KUNDE)
                .eq(StammkundePool::getShopId, shopId)
                .le(StammkundePool::getActivationTime, expire)
        );
    }

    private int defeatNum(Long userId, Long shopId, Date expire) {
        Timestamp endDay = DateUtil.getExpiredDay(expire, 180);
        return stammkundePoolService.count(Wrappers.<StammkundePool>lambdaQuery()
                .eq(StammkundePool::getAdviserId, userId)
                .eq(StammkundePool::getPoolStatus, StammkundeStatusEnum.KUNDE)
                .eq(StammkundePool::getShopId, shopId)
                .lt(StammkundePool::getActivationTime, expire)
                .le(StammkundePool::getRejectTime, endDay)

        );
    }

    private int initialNum(Long shopId, Date expire) {
        return stammkundePoolService.count(Wrappers.<StammkundePool>lambdaQuery()
                .eq(StammkundePool::getPoolStatus, StammkundeStatusEnum.KUNDE)
                .eq(StammkundePool::getShopId, shopId)
                .le(StammkundePool::getActivationTime, expire)
        );
    }

    private int defeatNum(Long shopId, Date expire) {
        Timestamp endDay = DateUtil.getExpiredDay(expire, 180);
        return stammkundePoolService.count(Wrappers.<StammkundePool>lambdaQuery()
                .eq(StammkundePool::getPoolStatus, StammkundeStatusEnum.KUNDE)
                .eq(StammkundePool::getShopId, shopId)
                .lt(StammkundePool::getActivationTime, expire)
                .le(StammkundePool::getRejectTime, endDay)
        );
    }
}
